## COPYRIGHT 2009 Hannes Isaak Reuter
# All rights reserved
# Contact: gisxperts@web.de
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
##/*---------------------------------------------------------------------------------------------------------
##/*  USDA Forest Service - Rocky Mountain Research Station - FSL. Moscow, ID
##/*---------------------------------------------------------------------------------------------------------
##/*  Program: LANDFORM.AML
##/*  Purpose: Calculate McNab or Bolstad concavity convexity landform index
##/*
##/*---------------------------------------------------------------------------------------------------------
##/*  Usage: LANDFORM <DEM> <OUTGRID> {MCNAB | BOLSTAD}
##/*
##/*  Arguments: DEM - Input DEM
##/*             OUTGRID - Out landform index.
##/*             METHOD - Caculation method for landform index. Default is McNab's method.
##/*---------------------------------------------------------------------------------------------------------
##/*  Notes:
##/*
##/*---------------------------------------------------------------------------------------------------------
##/*  Input:  Digital Elevation Model
##/*  Output: McNab's or Bolstad's landform index
##/*
##/*---------------------------------------------------------------------------------------------------------
##/*  History: Jeffrey Evans - Landscape Ecologist
##/*           09/18/02 - Original coding
##/*           1221 South Main, Moscow, ID 83843
##/*           (208) 882-3557
##/*           jevans02@fs.fed.us
##/*=========================================================================================================
##/*  References:
##/*
##/*           Bolstad, P.V., Swank, W., Vose J., 1998. Predicting Southern Applachian overstory vegetion
##/*           with digital terrain data.. Landscape Ecology. 13: 271-283.
##/*
##/*           McNab, H.W. 1989. Terrain shape index: quantifying effect of minor landforms on tree
##/*           height. Forest Science. 35(1): 91-104.
##/*
##/*           McNab, H.W. 1993. A topographic index to quantify the effect of mesoscale landform on
##/*           site productivity. Canadian Journal of Forest Research. 23: 1100-1107.
##/*
##/*=========================================================================================================
##/*C:\WorkSpace\pythontbx\a25m C:\WorkSpace\pythontbx\a25m_mcnab McNab
import sys, string, os, tempfile,  traceback
# Create the Geoprocessor object
version = sys.version[0:4]
if version == '2.1.':
    import win32com.client,tp21
    gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
elif version == '2.4.':
    import arcgisscripting
    import tp24 as tp
    gp = arcgisscripting.create()
elif version == '2.5.':
    import arcgisscripting
    import tp25 as tp
    gp = arcgisscripting.create()
elif version == '2.6.':
    import arcgisscripting
    import tp26 as tp
    gp = arcgisscripting.create()
else:
    raise Exception, 'Version not yet supported'

ARCGIS = ""
ARCINFO = ""
scratchWS = ""
# Script arguments...
Expression = sys.argv[1]
output = sys.argv[2]
method = sys.argv[3].upper()
# check for method
if not method in ('MCNAB','BOLSTAD'):
    raise Exception, "Method not specified or allowed - MCNAB or BOLSTAD"
# DEBUG = True
if len(sys.argv) >= 5:
    DEBUG = sys.argv[4]
    if DEBUG == '#':
        DEBUG = 0
else:
    DEBUG = 0
# setup
ARCGIS,ARCINFO,scratchWS = tp.geomorphometrysetup(gp,sys,os,DEBUG)
# create variables for workspace
dem,workspace = tp.getnameandworkspace_base(Expression,os,gp,string,DEBUG)
gp.workspace = workspace
dem_out,outworkspace = tp.checkoutput(output,os,gp)
# how to handle nodata
InNoDataOption = "DATA"
InNeighborhood = "Rectangle 3 3 CELL"

# set names
tmp12345 = gp.createscratchname("tmp55","","raster",scratchWS)
gp.CreateConstantRaster_sa(tmp12345, 0, "INTEGER",1,"0, 0, 2, 2")
tmp12346 = gp.createscratchname("tmp56","","raster",scratchWS)
gp.CreateConstantRaster_sa(tmp12346, 0, "INTEGER",1,"0, 0, 2, 2")
tmp12347 = gp.createscratchname("tmp57","","raster",scratchWS)
gp.CreateConstantRaster_sa(tmp12347, 0, "INTEGER",1,"0, 0, 2, 2")
tmp12348 = gp.createscratchname("tmp58","","raster",scratchWS)
gp.CreateConstantRaster_sa(tmp12348, 0, "INTEGER",1,"0, 0, 2, 2")

try:
# clean temp files
 # clean temp files
    tp.cleanttempfile4(gp,scratchWS,workspace,os)


# define windows smaller than the original values
# Set local variables
# Process: GetRasterProperties
# define windows smaller than the original values
# Set local variables
    gp.extent = tp.getextentminus1(dem,gp)

# Local variables...
    dem = Expression

    print " Process DEM "+dem+ " for method: "+method+' Landform Index'
    gp.AddMessage(" Process DEM "+dem+ " for method: "+method+' Landform Index')


    if method == 'BOLSTAD':

      gp.FocalStatistics_sa(dem, tmp12345, InNeighborhood, "MEAN", InNoDataOption)
      gp.minus_sa(dem,tmp12345, tmp12346)
      if gp.Exists(output):
            gp.delete_management(output)
      gp.divide_sa(tmp12346, 3.6200, output)
    elif method == 'MCNAB':
    ##/*=========================================================================================================
    ##/* McNab's landform Index
    ##/*=========================================================================================================
      tp.mcnabclassification(dem,output,gp,ARCINFO,workspace,tempfile,os)

    else:
              gp.AddMessage("Method not implemented: "+method)
              print "Method not implemented: "+method
    # final clean up
    gp.CheckInExtension("spatial")
     # so we see something if do error testing in idlde
    print "Finished landformshape.py ok for input "+dem+' for output: '+ dem_out
    gp.AddMessage("Finished landformshape.py ok for "+dem+' for output: '+ dem_out)
    # clean temp files
    if not DEBUG :
        tp.cleanttempfile4(gp,scratchWS,workspace,os)



except:
##/* ######################################################
##/* routine bailout start
##/* ######################################################
  tp.errorcall(sys,gp,traceback)